/*	Screen-Space Ambient Occlusion module.
Based on GSC build 3120 implementation.
Edited by K.D. (OGSE team)
13.11.2012
*/

#ifndef OGSE_SSAO_H
#define OGSE_SSAO_H
static const float3 poisson_disc12[12] = 
{
	float3(-0.326212f , -0.405810f, 0.f),
	float3(-0.840144f , -0.073580f, 0.f),
	float3(-0.695914f ,  0.457137f, 0.f),
	float3(-0.203345f ,  0.620716f, 0.f),
	float3( 0.962340f , -0.194983f, 0.f),
	float3( 0.473434f , -0.480026f, 0.f),
	float3( 0.519456f ,  0.767022f, 0.f),
	float3( 0.185461f , -0.893124f, 0.f),
	float3( 0.507431f ,  0.064425f, 0.f),
	float3( 0.896420f ,  0.412458f, 0.f),
	float3(-0.321940f , -0.932615f, 0.f),
	float3(-0.791559f , -0.597710f, 0.f)
};

float3 calc_ssao(float3 P, float3 N, float2 tc)
{
    const float minCrease   = -0.1;
    const float creaseScale = 1.0;
	const float distScale   = 0.5;
	int quality = SSAO_QUALITY + 1;
	float 	scale 	= 1.f + P.z / 8.f ;
	float occ = 0;
	int num_dir = 0;
	float c = 1;
	if(P.z<SSAO_FADE_DIST) c = SSAO_FADE_COEF + ((1-SSAO_FADE_COEF)*P.z)/SSAO_FADE_DIST;
	if (P.z < 0.001) P.z = 10000.h; //sky
	for (int a=1; a<quality; ++a)
	{
		float	scale_tmp = scale*a;
		for (int i=0; i<12; i++)
		{
			float3 	tap 	= P + (poisson_disc12[i] + N) * scale_tmp/50;  // try this - pseudo random
			float4 tap_scr = convert_to_screen_space(mul(m_P, float4(tap, 1.0)));
			float3	tap_pos	= tex2Dproj	(s_position,tap_scr);
			float3 	dir 	= tap_pos-P.xyz;
			float	dist	= length(dir);
			dir 	= normalize(dir);
/*			float 	infl 	= clamp(dot( dir, N.xyz),0,c);
			float 	occ_factor = saturate(dist);
			occ += (infl+0.01)*lerp( 1, occ_factor, infl)/(occ_factor+0.1);
			num_dir += (infl+0.01)/(occ_factor+0.1);*/
		float   normAtten  = clamp ( ( dot ( dir, N.xyz ) - minCrease ) * creaseScale, 0.0, c );
        float   rangeAtten = 1.0 - clamp ( dist / distScale, 0.0, 1.0 );

        occ += normAtten * rangeAtten;
		}
	}
//	return (occ / num_dir);

	return pow ( (1.0 - occ / 12.0 / SSAO_QUALITY)*1.1, 2.0 )*float3(1,1,1);
}
#endif