#ifndef OGSE_DOF_H
#define OGSE_DOF_H
#include "common.h"
#define EPSDEPTH 0.001
uniform half4 e_barrier;
uniform half4 e_weights;
uniform half4 e_kernel;
uniform half4 zoom_dof_params;
#ifndef DDOF_QUALITY
	#define	DDOF_MAX_SAMPLES int(1)
#else
	#if DDOF_QUALITY==1
		#define	DDOF_MAX_SAMPLES int(6)
	#else
		#if DDOF_QUALITY==2
			#define	DDOF_MAX_SAMPLES int(12)
		#else
			#if DDOF_QUALITY==3
				#define	DDOF_MAX_SAMPLES int(18)
			#endif
		#endif
	#endif
#endif
float3 classicblur(float2 center, float3 sum, float factor, int samples) {
	float2 bias=float2(.5f/1024.f,.5f/768.f);
	float2 scale = bias * factor;
	float2 o [48];
	o[0]=float2(-0.326212f,-0.405810f)*scale;
	o[1]=float2(-0.840144f,-0.073580f)*scale;
	o[2]=float2(-0.695914f,0.457137f)*scale;
	o[3]=float2(-0.203345f,0.620716f)*scale;
	o[4]=float2(0.962340f,-0.194983f)*scale;
	o[5]=float2(0.473434f,-0.480026f)*scale;
	o[6]=float2(0.519456f,0.767022f)*scale;
	o[7]=float2(0.185461f,-0.893124f)*scale;
	o[8]=float2(0.507431f,0.064425f)*scale;
	o[9]=float2(0.896420f,0.412458f)*scale;
	o[10]=float2(-0.321940f,-0.932615f)*scale;
	o[11]=float2(-0.791559f,-0.597710f)*scale;	
		
	float contrib=1.h;
	int j = 0;
	
	for (int i=0;i<samples;i++)
	{	
		float2 tap=center+o[j];
		float3 tap_color=tex2D(s_image,tap).rgb;
		sum+=tap_color;
		contrib++;
		j++; if (j > 11) { j=0; }
	}
	
	return sum;
}
float3 simpleblur(float2 center, float factor) {
   float dx = factor * .5f/1024.f;
   float dy = factor * .5f/768.f;   
   return   
	(
      1 * tex2D( s_image, center ).rgb +
      1 * tex2D( s_image, center + float2(dx, 0) ).rgb +
	  1 * tex2D( s_image, center + float2(dy, 0) ).rgb +
      1 * tex2D( s_image, center + float2(-dx, 0) ).rgb +      
      1 * tex2D( s_image, center + float2(-dx, -dy) ).rgb +
      1 * tex2D( s_image, center + float2(dx, -dy) ).rgb
	)/6;
}
float3 blur_dof(float2 center, float blur_mycof, float3 img) {

	float3 sum = img;
	float contrib = 1.h;

	#ifdef DOF_CLASSIC_ZOOM // use classic blur algorithm
		sum = classicblur(center, sum, blur_mycof, 12);
		contrib += 12;
	#endif
	#ifdef DDOF_GAUSSIAN // use gaussian blur algorithm
		float total_blur = blur_mycof*DDOF_CORRECTION;
		float f = 0.f;
		float inc = total_blur/DDOF_MAX_SAMPLES;
		for (int i=0;i<DDOF_MAX_SAMPLES;i++){sum+=simpleblur(center,f);contrib++;f+=inc;}
	#endif

	return (sum/contrib);	
}
float calc_regular_dof(float3 pos) {
	float dep = pos.z;
	float mycof = DDOF_MAXCOF_FAR;
	float myMINDIST = DDOF_MINDIST_COMMON;
	float myMAXDIST = DDOF_MAXDIST_COMMON;
	float myMAXNEAR = DDOF_MAXNEAR;
	float myMINNEAR = DDOF_MINNEAR;
	float targetDist = tex2D(s_position,float2(0.500001f,0.500001f)).z;

	if (targetDist <= myMAXNEAR) myMAXNEAR = targetDist;

	// Dynamic DOF
	if (targetDist < DDOF_MAXDIST && targetDist < DDOF_MAXBARRIER)
	{
		mycof = DDOF_MAXCOF;
		myMINDIST = targetDist + DDOF_MINDIST_ADD;	
		float maxFactor = (targetDist < DDOF_MAXDIST) ? 1 : (targetDist / DDOF_MAXDIST);
		myMAXDIST = myMINDIST * DDOF_MAXDIST_FACTOR * maxFactor;
	}
	
	// Near Field DOF
	if (dep <= myMAXNEAR && dep > myMINNEAR)
	{
		dep = (1 - (dep-myMINNEAR)/(myMAXNEAR-myMINNEAR) ) * myMAXDIST;
		mycof = DDOF_MAXCOF_NEAR;
	}
	else if (dep > myMAXNEAR && dep < targetDist-DDOF_MINDIST_ADD && dep > DDOF_MINDIST)
	{	
		dep = saturate( 1 - (dep-DDOF_MINDIST)/(targetDist-DDOF_MINDIST_ADD-DDOF_MINDIST) )  * DDOF_MAXDIST_COMMON;
		mycof = 0;
		myMAXDIST = DDOF_MAXDIST_COMMON;
	}
	else if (dep > myMAXNEAR && dep < targetDist-DDOF_MINDIST_ADD && dep <= DDOF_MINDIST)
	{
		dep = saturate( (dep - myMAXNEAR)/(DDOF_MINDIST - myMAXNEAR) ) * DDOF_MAXDIST_COMMON;
		mycof = DDOF_MAXCOF_NEAR_DDOF;
		myMAXDIST = DDOF_MAXDIST_COMMON;
	}
	float blur_mycof = pow(saturate((dep-myMINDIST)/(myMAXDIST-myMINDIST)),2)*mycof;
	return blur_mycof;
}
float calc_zoom_dof(float2 center, float3 pos) {
/*	float dep = pos.z;
	float mycof=ZOOM_MAXCOF;
	float myMINDIST = ZOOM_MINDIST;
	float myMAXDIST = ZOOM_MAXDIST;
	float myMAXNEAR = DDOF_MAXNEAR;
	float myMINNEAR = DDOF_MINNEAR;
	float targetDist = tex2D(s_position,float2(0.500001f,0.497f)).z; // e_weights.x * 1000;
	if (targetDist <= myMAXNEAR) myMAXNEAR = targetDist;

	float2 p = pos.xy/pos.z;

	float center_dist = length(p);	// absolute distance from screen center
	float dist_factor = center_dist * ZOOM_FACTOR / e_weights.y; // weighted distance factor

	if (dep < myMAXNEAR && dep > myMINNEAR) // && targetDist > myMAXNEAR)
	{
		dep = (1 - (dep-myMINNEAR)/(myMAXNEAR-myMINNEAR) ) * myMAXDIST;
		mycof = DDOF_MAXCOF_NEAR;
	}
	else
		dep = dist_factor;*/

	float myMINDIST = ZOOM_MINDIST;
	float myMAXDIST = ZOOM_MAXDIST;		
	float mycof = ZOOM_MAXCOF;
	float2 cent = center;
	float dep = length(cent - float2(0.500001f,0.497f));
	float blur_mycof = pow(saturate((dep-myMINDIST)/(myMAXDIST-myMINDIST)),2)*mycof;
	return blur_mycof;
}

float3 calc_dof(float2 center, float3 pos, float3 img) {
	#ifndef DDOF_QUALITY
		return img;
	#else
		float blur_mycof = calc_regular_dof(pos);
		blur_mycof *= 2;
		if (zoom_dof_params.x > 0.5) {
			blur_mycof /= 2;
			blur_mycof += calc_zoom_dof(center, pos);
		}
/*		if (zoom_dof_params.x > 0.5) {
			dep_mycof = calc_zoom_dof(center, pos);
		} else {
			dep_mycof = calc_regular_dof(pos);
		}*/
		return blur_dof(center, blur_mycof/2, img);	
	#endif
}
#endif