//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$//
//---  The below code is copyright of Sky4CE ---//
//----------- mailto: Sky4CE@inbox.ru -----------//
//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$//
#ifndef OGSE_POM_H
#define OGSE_POM_H
#ifndef STEEP_PARALLAX_QUALITY
	#define	POM_MAX_SAMPLES int(1)
	#define	POM_MIN_SAMPLES int(1)
	#define	POM_FINAL_INTERSECTION_LOOPS int (1)
#else
	#if STEEP_PARALLAX_QUALITY==1
		#define	POM_MAX_SAMPLES int(15)
		#define	POM_MIN_SAMPLES int(3)
		#define	POM_FINAL_INTERSECTION_LOOPS int (3)
	#else
		#if STEEP_PARALLAX_QUALITY==2
			#define	POM_MAX_SAMPLES int(30)
			#define	POM_MIN_SAMPLES int(5)
			#define	POM_FINAL_INTERSECTION_LOOPS int (5)
		#else
			#if STEEP_PARALLAX_QUALITY==3
				#define	POM_MAX_SAMPLES int(40)
				#define	POM_MIN_SAMPLES int(10)
				#define	POM_FINAL_INTERSECTION_LOOPS int (7)
			#endif
		#endif
	#endif
#endif
float2 calc_parallax(float3 eye,float2 tc,sampler2D s_base)
{
 	int nNumSteps = lerp( POM_MAX_SAMPLES, POM_MIN_SAMPLES, normalize(eye).z );
  	float2 vDelta = -normalize(eye).xy * POM_PARALLAX_OFFSET;
  	#ifdef POM_CORRECT_PERSPECTIVE
		vDelta /= normalize(eye).z;
  	#endif
   	float fStepSize = 1.0 / nNumSteps;
   	float2 vTexOffsetPerStep = fStepSize * vDelta;
   	double2 vTexCurrentOffset = tc;
   	float fCurrHeight = 0;
   	float  fCurrentBound = 1.0;
   	float Q1 = POM_CONTRAST_COEF_Q1;
   	float Q2 = POM_CONTRAST_COEF_Q2;
   	float Brightness = POM_BRIGHTNESS_COEF;
   	for(;fCurrHeight < fCurrentBound;fCurrentBound -= fStepSize)
   	{
   		vTexCurrentOffset.xy += vTexOffsetPerStep;
		fCurrHeight = tex2Dlod( s_base, float4(vTexCurrentOffset.xy,0,0) ).a+Brightness;
		if (fCurrHeight > Q2) fCurrHeight  = 1.0f;
		else if (fCurrHeight < Q1) fCurrHeight = 0.0f;
		else fCurrHeight = (fCurrHeight - Q1)/(Q2-Q1);
    }
    float4 offsetBest = float4(vTexCurrentOffset,0,0);
    vTexCurrentOffset.xy -= vTexOffsetPerStep;
	float fPrevHeight = tex2Dlod( s_base, float4(vTexCurrentOffset.xy,0,0) ).a+Brightness;
	if (fPrevHeight > Q2) fPrevHeight = 1.0f;
	else if (fPrevHeight < Q1) fPrevHeight = 0.0f;
	else fPrevHeight = (fPrevHeight - Q1)/(Q2-Q1);
    float error = 1.0;
    float t1 = fCurrentBound ;
    float t0 = t1 + fStepSize;
    float delta1 = t1 - fCurrHeight;
    float delta0 = t0 - fPrevHeight;
    float4 intersect = float4(vDelta, vDelta + tc);
    for (int i=0; i<POM_FINAL_INTERSECTION_LOOPS && abs(error) > 0.01; i++)
    { 
      float denom = (delta1 - delta0);
      float t = (t0 * delta1 - t1 * delta0) / denom;
      offsetBest.xy = -t * intersect.xy + intersect.zw;
	  float NB = tex2Dlod(s_base, offsetBest).a+Brightness;
	  if (NB > Q2) NB = 1.0f;
	  else if (NB < Q1) NB = 0.0f;
	  else NB = (NB - Q1)/(Q2-Q1);
      error = t - NB;
      if (error < 0)
      {
         delta1 = error;
         t1 = t;
      }
      else
      {
         delta0 = error;
         t0 = t;
      }
    }
    return offsetBest.xy;
}
#endif