/*	Screen-Space Directional Occlusion.
Based on Tobias Ritschel tech paper.
Implemented by K.D. (OGSE team)
Last Changed: 17.11.2012
*/
#ifndef OGSE_SSDO_H
#define OGSE_SSDO_H
#ifndef SSAO_QUALITY
	#define	SSAO_QUALITY int(1)
#endif
static const float3 arr[13] = 
{
	float3(	-0.840144	,	-0.07358	,	0	),
	float3(	-0.691559	,	-0.59771	,	-0.4	),
	float3(	-0.5		,	0.457137	,	-0.3	),
	float3(	-0.426212	,	-0.40581	,	-0.7	),
	float3(	-0.32194	,	-0.932615	,	-0.5	),
	float3(	-0.203345	,	0.620716	,	-1	),
	float3(	-0.1		,	-0.4		,	-0.1	),
	float3(	0.185461	,	-0.893124	,	-0.5	),
	float3(	0.37		,	-0.480026	,	-0.8	),
	float3(	0.507431	,	0.064425	,	-0.2	),
	float3(	0.61		,	0.767022	,	-0.5	),
	float3(	0.89642		,	0.412458	,	-0.2	),
	float3(	0.96234		,	-0.194983	,	0	)
};
uniform sampler2D s_smap_surf;
float3 calc_ssdo(float3 P, float3 N, float2 tc)
{
	int quality = SSAO_QUALITY + 1;
	float3 occ = float3(0,0,0);
	float c = 1;
	P.z += 10000000.h * saturate(0.001 - P.z); //sky
	float scale = P.z/10.f * SSDO_RADIUS;
	for (int a=1; a<quality; ++a)
	{
		scale *= a;
		for (int i=0; i<13; i++)
		{
			float3 occ_pos_view = P + (arr[i] + N) * scale;
			float4 occ_pos_screen = convert_to_screen_space(mul(m_P, float4(occ_pos_view, 1.0)));
			float3 screen_occ = tex2Dproj(s_position, occ_pos_screen);
			// consider only unoccluded samples
			float occ_coeff = saturate((screen_occ.z - occ_pos_view.z)*100);
			// fix weapon occlusion
			occ_coeff += saturate((2.1 - screen_occ.z));
			occ += float3(occ_coeff, occ_coeff, occ_coeff);
/*			#ifdef SSDO_USE_INDIRECT_BOUNCES
				// collect also indirect bonuses. Must be discarded:
				// - samples outside testing hemisphere
				// - backfacing samples
				float3 sample_normal = tex2Dproj(s_normal, occ_pos_screen);
				float3 sample_diffuse = tex2Dproj(s_diffuse, occ_pos_screen);
				float3 dir = occ_pos_view - P;
				float dist = saturate(dot(dir, dir));
				occ += saturate((P.z - screen_occ.z)*100) * saturate(-dot(sample_normal, N)) * sample_diffuse * SSDO_COLOR_BLEEDING;
			#endif*/
		}
	}
	occ /= (13 * SSAO_QUALITY);
	occ = saturate(occ);
	return (occ + (1 - occ)*(1 - SSDO_BLEND_FACTOR));
}
#endif